{ File   : IMAHeader.pas
  Author : Deniz Oezmen
  Created: 2007-07-23
  Changed: 2010-08-29

  Import header for IMAMgr.dll
}
unit IMAHeader;

interface

const
  // header version
  VersionMajor = 1;
  VersionMinor = 10;
  VersionBuild = 4429;

  // list of compatible DLL versions
  CompatBuildList: array[0..1] of record
                                    Major,
                                    Minor,
                                    Build: Longword;
                                  end
                 = ((Major: 1; Minor: 10; Build: 0804),
                    (Major: 1; Minor: 10; Build: 2128));

  // name of compiled IMA manager DLL
  DLLName = 'IMAMgr.dll';

  // library return value error constants
  Handler_Success       = Longint($00000000);
  Handler_InvalidHandle = Longint($80010001);
  Handler_OutOfMemory   = Longint($80010002);

  // interface format constants
  IMA_ADPCM_Format_BS3    : TGUID = '{870AF4DB-8CD5-4FF8-B5FA-BD2C8C4223B9}';
  IMA_ADPCM_Format_Default: TGUID = '{CCF8D462-B5BE-4CD7-AFEA-85A910AB58A2}';
  IMA_ADPCM_Format_Driver : TGUID = '{472A60AB-896D-4E93-B769-59331647308D}';
  IMA_ADPCM_Format_Express: TGUID = '{6E4357BA-1D15-4EE5-96DB-D7C60391518E}';
  IMA_ADPCM_Format_Jack   : TGUID = '{6259E1BC-2212-4684-9422-3B982E10BC4C}';
  IMA_ADPCM_Format_TLJ    : TGUID = '{24C8C90B-D945-4F5D-996A-15A9EDB46783}';

type
  // method prototypes
  TGetIMAManager = function (BlockSize, SampPerBlock, NumChannels: Longint;
    const FormatType: TGUID): Longint; stdcall;
  TReleaseIMAManager = procedure (Handle: Longint); stdcall;
  TIMALoadSamples =  procedure (Handle: Longint; const InBuffer;
    NumSamples: Longint); stdcall;
  TIMAUnloadSamples = procedure (Handle: Longint); stdcall;
  TIMALoadCodes = procedure (Handle: Longint; const InBuffer;
    CodeSize: Longint); stdcall;
  TIMAUnloadCodes = procedure (Handle: Longint); stdcall;
  TIMAGetCompBlockSize = function (Handle: Longint): Longint; stdcall;
  TIMAGetDecompBlockSize = function (Handle: Longint): Longint; stdcall;
  TIMAGetCompressedSize = function (Handle: Longint): Longint; stdcall;
  TIMAGetDecompressedSize = function (Handle: Longint): Longint; stdcall;
  TIMAGetCompSizeBySize = function(Handle: Longint;
    NumSamples: Longint): Longint; stdcall;
  TIMAGetDecompSizeBySize = function(Handle: Longint;
    CodeSize: Longint): Longint; stdcall;
  TIMACompressBlock = function (Handle: Longint; var OutBuffer): Longint;
    stdcall;
  TIMADecompressBlock = function (Handle: Longint; var OutBuffer): Longint;
    stdcall;
  TIMACompress = procedure (Handle: Longint; var OutBuffer); stdcall;
  TIMADecompress = procedure (Handle: Longint; var OutBuffer); stdcall;
  TGetDLLVersion = procedure(var Major, Minor, Build: Longword); stdcall;

var
  // DLL import methods
  GetIMAManager         : TGetIMAManager;
  ReleaseIMAManager     : TReleaseIMAManager;
  IMALoadSamples        : TIMALoadSamples;
  IMAUnloadSamples      : TIMAUnloadSamples;
  IMALoadCodes          : TIMALoadCodes;
  IMAUnloadCodes        : TIMAUnloadCodes;
  IMAGetCompBlockSize   : TIMAGetCompBlockSize;
  IMAGetDecompBlockSize : TIMAGetDecompBlockSize;
  IMAGetCompressedSize  : TIMAGetCompressedSize;
  IMAGetDecompressedSize: TIMAGetDecompressedSize;
  IMAGetCompSizeBySize  : TIMAGetCompSizeBySize;
  IMAGetDecompSizeBySize: TIMAGetDecompSizeBySize;
  IMACompressBlock      : TIMACompressBlock;
  IMADecompressBlock    : TIMADecompressBlock;
  IMACompress           : TIMACompress;
  IMADecompress         : TIMADecompress;
  GetDLLVersion         : TGetDLLVersion;

(* header file exports ********************************************************)

function LoadDLL(LoadPath: AnsiString = ''): Boolean;

procedure UnloadDLL;

function DLLCanUse: Boolean;

function GetErrorName(ErrorCode: Longint): AnsiString;

implementation

uses
  SysUtils, Windows;

type
  TIsHeaderSupported =
    function (ExtMajor, ExtMinor, ExtBuild: Longword): Boolean;
    stdcall;

var
  IsHeaderSupported: TIsHeaderSupported;
  DLLHandle        : THandle;

(* header file export implementations *****************************************)

function LoadDLL(LoadPath: AnsiString = ''): Boolean;
begin
  if LoadPath = '' then
    DLLHandle := LoadLibrary(DLLName)
  else
    DLLHandle := LoadLibrary(PAnsiChar(ExpandUNCFileName(
      IncludeTrailingPathDelimiter(LoadPath) + DLLName)));

  Result := (DLLHandle <> 0);
  if not Result then
    Exit;

  GetIMAManager          := GetProcAddress(DLLHandle, 'GetIMAManager');
  Result                 := Result and Assigned(GetIMAManager);
  ReleaseIMAManager      := GetProcAddress(DLLHandle, 'ReleaseIMAManager');
  Result                 := Result and Assigned(ReleaseIMAManager);
  IMALoadSamples         := GetProcAddress(DLLHandle, 'IMALoadSamples');
  Result                 := Result and Assigned(IMALoadSamples);
  IMAUnloadSamples       := GetProcAddress(DLLHandle, 'IMAUnloadSamples');
  Result                 := Result and Assigned(IMAUnloadSamples);
  IMALoadCodes           := GetProcAddress(DLLHandle, 'IMALoadCodes');
  Result                 := Result and Assigned(IMALoadCodes);
  IMAUnloadCodes         := GetProcAddress(DLLHandle, 'IMAUnloadCodes');
  Result                 := Result and Assigned(IMAUnloadCodes);
  IMAGetCompBlockSize    := GetProcAddress(DLLHandle, 'IMAGetCompBlockSize');
  Result                 := Result and Assigned(IMAGetCompBlockSize);
  IMAGetDecompBlockSize  := GetProcAddress(DLLHandle, 'IMAGetDecompBlockSize');
  Result                 := Result and Assigned(IMAGetDecompBlockSize);
  IMAGetCompressedSize   := GetProcAddress(DLLHandle, 'IMAGetCompressedSize');
  Result                 := Result and Assigned(IMAGetCompressedSize);
  IMAGetDecompressedSize := GetProcAddress(DLLHandle, 'IMAGetDecompressedSize');
  Result                 := Result and Assigned(IMAGetDecompressedSize);
  IMAGetCompSizeBySize   := GetProcAddress(DLLHandle, 'IMAGetCompSizeBySize');
  Result                 := Result and Assigned(IMAGetCompSizeBySize);
  IMAGetDecompSizeBySize := GetProcAddress(DLLHandle, 'IMAGetDecompSizeBySize');
  Result                 := Result and Assigned(IMAGetDecompSizeBySize);
  IMACompressBlock       := GetProcAddress(DLLHandle, 'IMACompressBlock');
  Result                 := Result and Assigned(IMACompressBlock);
  IMADecompressBlock     := GetProcAddress(DLLHandle, 'IMADecompressBlock');
  Result                 := Result and Assigned(IMADecompressBlock);
  IMACompress            := GetProcAddress(DLLHandle, 'IMACompress');
  Result                 := Result and Assigned(IMACompress);
  IMADecompress          := GetProcAddress(DLLHandle, 'IMADecompress');
  Result                 := Result and Assigned(IMADecompress);
  IsHeaderSupported      := GetProcAddress(DLLHandle, 'IsHeaderSupported');
  Result                 := Result and Assigned(IsHeaderSupported);
  GetDLLVersion          := GetProcAddress(DLLHandle, 'GetDLLVersion');
  Result                 := Result and Assigned(GetDLLVersion)
end;

procedure UnloadDLL;
begin
  if DLLHandle <> 0 then
    FreeLibrary(DLLHandle)
end;

function IsDLLSupported(ExtMajor, ExtMinor, ExtBuild: Longword): Boolean;
var
  i: Longint;
begin
  Result := False;

  for i := 0 to High(CompatBuildList) do
    with CompatBuildList[i] do
      Result := Result or
        ((ExtMajor = Major) and (ExtMinor = Minor) and (ExtBuild = Build))
end;

function DLLCanUse: Boolean;
var
  DLLMajor,
  DLLMinor,
  DLLBuild: Longword;
begin
  GetDLLVersion(DLLMajor, DLLMinor, DLLBuild);

  Result := IsHeaderSupported(VersionMajor, VersionMinor, VersionBuild) or
    IsDLLSupported(DLLMajor, DLLMinor, DLLBuild)
end;

function GetErrorName(ErrorCode: Longint): AnsiString;
begin
  case ErrorCode of
    Handler_Success:
      Result := 'Handler_Success';
    Handler_InvalidHandle:
      Result := 'Handler_InvalidHandle';
    Handler_OutOfMemory:
      Result := 'Handler_OutOfMemory'
  else
    Result := 'unknown error code'
  end
end;

end.


