{ File   : Utils.pas
  Author : Deniz Oezmen
  Created: 2008-08-21
  Changed: 2008-08-21

  Memory search utilities.
}
unit Utils;

interface

function SearchMem(Pattern, Data: Pointer;
  PatternSize, DataSize: Longint): Longint;
function SearchMemReverse(Pattern, Data: Pointer;
  PatternSize, DataSize: Longint): Longint;

implementation

uses
  SysUtils;

// Rabin-Karp search algorithm as presented in Sedgewick, Algorithms, Chapter 19
function SearchMem(Pattern, Data: Pointer;
  PatternSize, DataSize: Longint): Longint;
const
  q = 8388593; // largest prime such that q * 256 <= 2^31 - 1
type
  PByte = ^Byte;
var
  i,
  N, M      : Longint;
  h1, h2,
  dM        : Longword;
  PatternPtr,
  DataPtr   : PByte;
begin
  Result := -1;

  N := DataSize;
  M := PatternSize;

  dM := 1;
  for i := 1 to M - 1 do
    dM := (dM shl 8) mod q;

  h1 := 0;
  PatternPtr := Pattern;
  for i := 1 to M do
  begin
    h1 := (h1 shl 8 + PatternPtr^) mod q;
    Inc(PatternPtr)
  end;

  h2 := 0;
  DataPtr := Data;
  for i := 1 to M do
  begin
    h2 := (h2 shl 8 + DataPtr^) mod q;
    Inc(DataPtr)
  end;

  i := 0;
  DataPtr := Data;
  PatternPtr := Pattern;
  repeat
    if (h1 = h2) and (CompareMem(DataPtr, PatternPtr, M)) then
    begin
      Result := i;
      Break
    end;

    h2 := (h2 + q shl 8 - DataPtr^ * dM) mod q;
    h2 := (h2 shl 8 + PByte(Longint(DataPtr) + M)^) mod q;

    Inc(i);
    Inc(DataPtr)
  until i > N - M
end;

// same algorithm as above, but searches through Data in reverse order,
// effectively returning the *last* occurrence of the search pattern
function SearchMemReverse(Pattern, Data: Pointer;
  PatternSize, DataSize: Longint): Longint;
const
  q = 8388593; // largest prime such that q * 256 <= 2^31 - 1
type
  PByte = ^Byte;
var
  i,
  N, M      : Longint;
  h1, h2,
  dM        : Longword;
  PatternPtr,
  DataPtr   : PByte;
begin
  Result := -1;

  N := DataSize;
  M := PatternSize;

  dM := 1;
  for i := 1 to M - 1 do
    dM := (dM shl 8) mod q;

  h1 := 0;
  PatternPtr := PByte(Longint(Pattern) + M - 1);
  for i := 1 to M do
  begin
    h1 := (h1 shl 8 + PatternPtr^) mod q;
    Dec(PatternPtr)
  end;

  h2 := 0;
  DataPtr := PByte(Longint(Data) + N - 1);
  for i := 1 to M do
  begin
    h2 := (h2 shl 8 + DataPtr^) mod q;
    Dec(DataPtr)
  end;

  i := N - M;
  DataPtr := PByte(Longint(Data) + N - 1);
  PatternPtr := Pattern;
  repeat
    if (h1 = h2) and (CompareMem(PByte(Longint(DataPtr) - M + 1), PatternPtr,
      M)) then
    begin
      Result := i;
      Break
    end;

    h2 := (h2 + q shl 8 - DataPtr^ * dM) mod q;
    h2 := (h2 shl 8 + PByte(Longint(DataPtr) - M)^) mod q;

    Dec(i);
    Dec(DataPtr)
  until i < 0
end;

end.
 